]> the.earth.li Git - onak.git/blobdiff - keydb.c
Create an initial database when installing the package.
[onak.git] / keydb.c
diff --git a/keydb.c b/keydb.c
index c5b07c628dfd2040d3152311dc85bec31ce729ad..2b419ca2af99d162643448489a0f9850de665651 100644 (file)
--- a/keydb.c
+++ b/keydb.c
 
 #include <stdio.h>
 
+#include "decodekey.h"
+#include "hash.h"
 #include "keydb.h"
 #include "keyid.h"
-#include "keyindex.h"
 #include "keystructs.h"
 #include "mem.h"
 #include "parsekey.h"
@@ -60,11 +61,13 @@ char *keyid2uid(uint64_t keyid)
 /**
  *     getkeysigs - Gets a linked list of the signatures on a key.
  *     @keyid: The keyid to get the sigs for.
+ *     @revoked: Is the key revoked?
  *
  *     This function gets the list of signatures on a key. Used for key 
*     indexing and doing stats bits.
  */
-struct ll *getkeysigs(uint64_t keyid)
+struct ll *getkeysigs(uint64_t keyid, bool *revoked)
 {
        struct ll *sigs = NULL;
        struct openpgp_signedpacket_list *uids = NULL;
@@ -76,6 +79,9 @@ struct ll *getkeysigs(uint64_t keyid)
                for (uids = publickey->uids; uids != NULL; uids = uids->next) {
                        sigs = keysigs(sigs, uids->sigs);
                }
+               if (revoked != NULL) {
+                       *revoked = (publickey->revocations != NULL);
+               }
                free_publickey(publickey);
        }
 
@@ -83,13 +89,48 @@ struct ll *getkeysigs(uint64_t keyid)
 }
 #endif
 
+/**
+ *     cached_getkeysigs - Gets the signatures on a key.
+ *     @keyid: The key we want the signatures for.
+ *     
+ *     This function gets the signatures on a key. It's the same as the
+ *     getkeysigs function above except we use the hash module to cache the
+ *     data so if we need it again it's already loaded.
+ */
+struct ll *cached_getkeysigs(uint64_t keyid)
+{
+       struct stats_key *key = NULL;
+       struct stats_key *signedkey = NULL;
+       struct ll        *cursig = NULL;
+       bool              revoked = false;
+
+       if (keyid == 0)  {
+               return NULL;
+       }
+
+       key = createandaddtohash(keyid);
+
+       if (key->gotsigs == false) {
+               key->sigs = getkeysigs(key->keyid, &revoked);
+               key->revoked = revoked;
+               for (cursig = key->sigs; cursig != NULL;
+                               cursig = cursig->next) {
+                       signedkey = (struct stats_key *) cursig->object;
+                       signedkey->signs = lladd(signedkey->signs, key);
+               }
+               key->gotsigs = true;
+       }
+
+       return key->sigs;
+}
+
 #ifdef NEED_GETFULLKEYID
 /**
  *     getfullkeyid - Maps a 32bit key id to a 64bit one.
  *     @keyid: The 32bit keyid.
  *
  *     This function maps a 32bit key id to the full 64bit one. It returns the
- *     full keyid.
+ *     full keyid. If the key isn't found a keyid of 0 is returned.
  */
 uint64_t getfullkeyid(uint64_t keyid)
 {
@@ -97,8 +138,13 @@ uint64_t getfullkeyid(uint64_t keyid)
 
        if (keyid < 0x100000000LL) {
                fetch_key(keyid, &publickey, false);
-               keyid = get_keyid(publickey);
-               free_publickey(publickey);
+               if (publickey != NULL) {
+                       keyid = get_keyid(publickey);
+                       free_publickey(publickey);
+                       publickey = NULL;
+               } else {
+                       keyid = 0;
+               }
        }
        
        return keyid;
@@ -60,11 +61,13 @@ char *keyid2uid(uint64_t keyid)
  *
  *     This function gets the list of signatures on a key. Used for key 
- *     indexing and doing stats bits.
+ *     indexing and doing stats bits. If revoked is non-NULL then if the key
+ *     is revoked it's set to true.
  */